@@ -7,7 +7,7 @@ from ast import Index
 from contextlib import suppress
 from importlib.util import find_spec
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, cast
 
 from flake8_type_checking.codes import TC001, TC002, TC003, TC004, TC005, TC100, TC101, TC200, TC201
 
@@ -17,18 +17,22 @@ with suppress(ModuleNotFoundError):
 
     # TODO: Find a better solution for this
     # The problem is essentially that we're triggering these errors in Django projects
-    # when running the code in _import_is_local. Perhaps user's could pass a map of which errors
+    # when running the code in _import_is_local. Perhaps users could pass a map of which errors
     # to handle and which values to return
     possible_local_errors += (AppRegistryNotReady, ImproperlyConfigured)
 
 if TYPE_CHECKING:
+    from _ast import AsyncFunctionDef, FunctionDef
     from argparse import Namespace
-    from typing import Any, Generator, Optional, Tuple, Union
+    from typing import Any, Optional, Union
 
-    from flake8_type_checking.types import ErrorDict, FunctionRangesDict, FunctionScopeImportsDict
-
-    ImportType = Union[ast.Import, ast.ImportFrom]
-    Flake8Generator = Generator[Tuple[int, int, str, Any], None, None]
+    from flake8_type_checking.types import (
+        ErrorDict,
+        Flake8Generator,
+        FunctionRangesDict,
+        FunctionScopeImportsDict,
+        Import,
+    )
 
 ATTRIBUTE_PROPERTY = 'flake8-type-checking_parent'
 
@@ -38,8 +42,15 @@ py38 = sys.version_info.major == 3 and sys.version_info.minor == 8
 class ImportVisitor(ast.NodeTransformer):
     """Map all imports outside of type-checking blocks."""
 
-    def __init__(self, cwd: Path, pydantic_enabled: bool, exempt_modules: Optional[list[str]] = None) -> None:
+    def __init__(
+        self,
+        cwd: Path,
+        pydantic_enabled: bool,
+        fastapi_enabled: bool,
+        exempt_modules: Optional[list[str]] = None,
+    ) -> None:
         self.pydantic_enabled = pydantic_enabled
+        self.fastapi_enabled = fastapi_enabled
         self.cwd = cwd  # we need to know the current directory to guess at which imports are remote and which are not
 
         # Import patterns we want to avoid mapping
@@ -57,10 +68,10 @@ class ImportVisitor(ast.NodeTransformer):
         self.uses: dict[str, ast.AST] = {}
 
         # Tuple of (node, import name) for all import defined within a type-checking block
-        self.type_checking_block_imports: set[tuple[ImportType, str]] = set()
+        self.type_checking_block_imports: set[tuple[Import, str]] = set()
         self.class_names: set[str] = set()
 
-        self.unused_type_checking_block_imports: set[tuple[ImportType, str]] = set()
+        self.unused_type_checking_block_imports: set[tuple[Import, str]] = set()
 
         # All type annotations in the file, without quotes around them
         self.unwrapped_annotations: list[tuple[int, int, str]] = []
@@ -196,7 +207,7 @@ class ImportVisitor(ast.NodeTransformer):
         origin = Path(spec.origin)
         return self.cwd in origin.parents
 
-    def _add_import(self, node: ImportType) -> None:
+    def _add_import(self, node: Import) -> None:
         """
         Add relevant ast objects to import lists.
 
@@ -204,8 +215,8 @@ class ImportVisitor(ast.NodeTransformer):
         """
         if self._in_type_checking_block(node):
             # For type checking blocks we want to
-            # 1. Record annotations for TCH2XX errors
-            # 2. Avoid recording imports for TCH1XX errors, by returning early
+            # 1. Record annotations for TC2XX errors
+            # 2. Avoid recording imports for TC1XX errors, by returning early
             for name_node in node.names:
                 if hasattr(name_node, 'asname') and name_node.asname:
                     name = name_node.asname
@@ -386,38 +397,84 @@ class ImportVisitor(ast.NodeTransformer):
         if getattr(node, 'value', None):
             self.generic_visit(node.value)  # type: ignore[arg-type]
 
-    def visit_FunctionDef(self, node: Union[ast.FunctionDef, ast.AsyncFunctionDef]) -> None:
-        """Remove and map function arguments and returns."""
-        # Map annotations
+    def _handle_fastapi_decorator(self, node: Union[AsyncFunctionDef, FunctionDef]) -> None:
+        """
+        Adjust for FastAPI decorator setting.
+
+        When the FastAPI decorator setting is enabled, treat all annotations
+        from within a function definition (except for return annotations) as
+        needed at runtime. To achieve this, visit the annotations to register them as "uses".
+        """
+        for path in [node.args.args, node.args.kwonlyargs]:
+            for argument in path:
+                if hasattr(argument, 'annotation') and argument.annotation:
+                    self.visit(argument.annotation)
+        if (
+            hasattr(node.args, 'kwarg')
+            and node.args.kwarg
+            and hasattr(node.args.kwarg, 'annotation')
+            and node.args.kwarg.annotation
+        ):
+            self.visit(node.args.kwarg.annotation)
+        if (
+            hasattr(node.args, 'vararg')
+            and node.args.vararg
+            and hasattr(node.args.vararg, 'annotation')
+            and node.args.vararg.annotation
+        ):
+            self.visit(node.args.vararg.annotation)
+
+    def _register_function_annotations(self, node: Union[AsyncFunctionDef, FunctionDef]) -> None:
         for path in [node.args.args, node.args.kwonlyargs]:
             argument: ast.arg
             for argument in path:
                 if hasattr(argument, 'annotation'):
                     self._add_annotation(argument.annotation)  # type: ignore[arg-type]
                     delattr(argument, 'annotation')
-        if hasattr(node.args, 'kwarg'):
-            kwarg = node.args.kwarg
-            if hasattr(kwarg, 'annotation'):
-                self._add_annotation(kwarg.annotation)  # type: ignore[arg-type,union-attr]
-                delattr(kwarg, 'annotation')
-        if hasattr(node.args, 'vararg'):
-            vararg = node.args.vararg
-            if hasattr(vararg, 'annotation'):
-                self._add_annotation(vararg.annotation)  # type: ignore[arg-type,union-attr]
-                delattr(vararg, 'annotation')
-        if hasattr(node, 'returns'):
-            self._add_annotation(node.returns)  # type: ignore[arg-type,union-attr]
+        if (
+            hasattr(node.args, 'kwarg')
+            and node.args.kwarg
+            and hasattr(node.args.kwarg, 'annotation')
+            and node.args.kwarg.annotation
+        ):
+            self._add_annotation(node.args.kwarg.annotation)
+            delattr(node.args.kwarg, 'annotation')
+        if (
+            hasattr(node.args, 'vararg')
+            and node.args.vararg
+            and hasattr(node.args.vararg, 'annotation')
+            and node.args.vararg.annotation
+        ):
+            self._add_annotation(node.args.vararg.annotation)
+            delattr(node.args.vararg, 'annotation')
+        if hasattr(node, 'returns') and node.returns:
+            self._add_annotation(node.returns)
             delattr(node, 'returns')
 
         # Register function start and end
-        for i in range(node.lineno, node.end_lineno + 1):  # type: ignore[arg-type,union-attr,operator]
-            self.function_ranges[i] = {'start': node.lineno, 'end': node.end_lineno + 1}  # type: ignore[operator]
+        end_lineno = cast(int, node.end_lineno)
+        for i in range(node.lineno, end_lineno + 1):
+            self.function_ranges[i] = {'start': node.lineno, 'end': end_lineno + 1}
+
+    def visit_FunctionDef(self, node: Union[ast.FunctionDef, ast.AsyncFunctionDef]) -> None:
+        """Remove and map function arguments and returns."""
+        if self.fastapi_enabled:
+            self._handle_fastapi_decorator(node)
+
+        # Map annotations
+        self._register_function_annotations(node)
 
         self.generic_visit(node)
 
     def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef) -> None:
         """Remove and map function arguments and returns."""
-        self.visit_FunctionDef(node)
+        if self.fastapi_enabled:
+            self._handle_fastapi_decorator(node)
+
+        # Map annotations
+        self._register_function_annotations(node)
+
+        self.generic_visit(node)
 
 
 class TypingOnlyImportsChecker:
@@ -435,8 +492,18 @@ class TypingOnlyImportsChecker:
 
         exempt_modules = getattr(options, 'type_checking_exempt_modules', [])
         pydantic_enabled = getattr(options, 'type_checking_pydantic_enabled', False)
+        fastapi_enabled = getattr(options, 'type_checking_fastapi_enabled', False)
 
-        self.visitor = ImportVisitor(self.cwd, pydantic_enabled=pydantic_enabled, exempt_modules=exempt_modules)
+        if fastapi_enabled and not pydantic_enabled:
+            # FastAPI support must include Pydantic support.
+            pydantic_enabled = True
+
+        self.visitor = ImportVisitor(
+            self.cwd,
+            pydantic_enabled=pydantic_enabled,
+            fastapi_enabled=fastapi_enabled,
+            exempt_modules=exempt_modules,
+        )
         self.visitor.visit(node)
 
         self.generators = [
